# coding=utf-8
from unittest import TestCase
from mock import Mock
import six
from earthdiagnostics.work_manager import Downloader, WorkManager
from earthdiagnostics.diagnostic import Diagnostic, DiagnosticStatus
from earthdiagnostics.datafile import DataFile, StorageStatus, LocalStatus
from bscearth.utils import log


class TestDownloader(TestCase):

    def setUp(self):
        self.downloader = Downloader()

    def test_start_and_stop(self):
        self.downloader.start()
        self.assertTrue(self.downloader._thread.is_alive())
        self.downloader.shutdown()
        self.assertFalse(self.downloader._thread.is_alive())

    def test_submit(self):
        datafile = Mock()
        self.downloader.submit(datafile)

    def test_download(self):
        datafile = Mock()
        self.downloader.submit(datafile)
        self.downloader.start()
        self.downloader.shutdown()

        datafile.download.assert_called_once()


class MockFile(DataFile):

    def download(self):
        self.local_status = LocalStatus.READY

    def upload(self):
        self.storage_status = StorageStatus.READY

    def clean_local(self):
        pass


class TestWorkManager(TestCase):

    def setUp(self):
        self.config = Mock()
        self.data_manager = Mock()
        self.config.max_cores = 1
        self.config.parallel_uploads = 1
        self.data_manager.requested_files = {}
        self.data_manager.request_chunk = self._request_chunk
        self.data_manager.declare_chunk = self._declare_chunk
        self.work_manager = WorkManager(self.config, self.data_manager)

    def test_prepare_job_list(self):

        class Diag1(Diagnostic):
            alias = 'diag1'

            @classmethod
            def generate_jobs(cls, diags, options):
                cls.process_options(options, [])
                diag = Diag1(self.data_manager)
                diag.add_subjob(Diag1(self.data_manager))
                return (diag,)

        self.config.get_commands.return_value = ['diag1', 'baddiag', 'diag1,badoption']
        Diagnostic.register(Diag1)
        self.work_manager.prepare_job_list()

    def test_run(self):
        self.config.max_cores = -1

        class EmptyDiag(Diagnostic):
            alias = 'diag1'

            def __str__(self):
                return 'Null diag'

            def compute(self):
                pass

            def request_data(self):
                pass

            def declare_data_generated(self):
                pass

        self.work_manager.jobs = [EmptyDiag(self.data_manager)]
        self.work_manager.run()

        for diag in self.work_manager.jobs:
            self.assertEqual(diag.status, DiagnosticStatus.COMPLETED)

    def test_failed_run(self):
        class FailDiag(Diagnostic):
            alias = 'diag1'

            def __str__(self):
                return 'Fail diag'

            def compute(self):
                raise Exception('Must fail')

            def request_data(self):
                pass

            def declare_data_generated(self):
                pass

        self.work_manager.jobs = [FailDiag(self.data_manager)]
        self.work_manager.run()

        for diag in self.work_manager.jobs:
            self.assertEqual(diag.status, DiagnosticStatus.FAILED)

    def _request_chunk(self, *args, **kwargs):
        req = MockFile()
        req.storage_status = StorageStatus.READY
        req.local_status = LocalStatus.PENDING
        req.remote_file = 'requested'
        self.data_manager.requested_files[req.remote_file] = req
        return req

    def _declare_chunk(self, *args, **kwargs):
        req = MockFile()
        req.remote_file = 'generated'
        self.data_manager.requested_files[req.remote_file] = req
        req.storage_status = StorageStatus.PENDING
        req.local_status = LocalStatus.NOT_REQUESTED
        return req

    def test_run_data(self):
        self.data_manager.config.max_cores = -1

        class DataDiag(Diagnostic):
            alias = 'diag1'

            def __str__(self):
                return 'Data diag'

            def compute(self):
                self.declared.local_status = LocalStatus.READY

            def request_data(self):
                self.request = self.request_chunk(None, None, None, None, None)

            def declare_data_generated(self):
                self.declared = self.declare_chunk(None, None, None, None, None)

        self.work_manager.jobs = [DataDiag(self.data_manager)]
        self.work_manager.run()

        for diag in self.work_manager.jobs:
            self.assertEqual(diag.status, DiagnosticStatus.COMPLETED)

    def test_run_empty(self):
        self.work_manager.jobs = []
        if six.PY3:
            with self.assertLogs(log.Log.log) as cmd:
                self.work_manager.run()
            self.assertTrue([record for record in cmd.records if
                             record.message == 'No diagnostics to run'])
        else:
            self.work_manager.run()
